@@ -13,7 +13,7 @@ use Scalar::Util 'weaken';
 
 use constant DEBUG => $ENV{MOJO_REACTOR_IOASYNC_DEBUG} || 0;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 my $IOAsync;
 
@@ -41,6 +41,13 @@ sub is_running { !!shift->{running} }
 # We have to fall back to Mojo::Reactor::Poll, since IO::Async::Loop is unique
 sub new { $IOAsync++ ? Mojo::Reactor::Poll->new : shift->SUPER::new }
 
+sub next_tick {
+	my ($self, $cb) = @_;
+	push @{$self->{next_tick}}, $cb;
+	$self->{next_timer} //= $self->timer(0 => \&_next);
+	return undef;
+}
+
 sub one_tick {
 	my $self = shift;
 	
@@ -85,7 +92,7 @@ sub reset {
 	$_->remove_from_parent for
 		grep { defined } map { delete $_->{watcher} }
 		values %{$self->{io}}, values %{$self->{timers}};
-	delete @{$self}{qw(io timers)};
+	delete @{$self}{qw(io next_tick next_timer timers)};
 }
 
 sub start {
@@ -112,10 +119,12 @@ sub watch {
 		$w->want_writeready($write);
 	} else {
 		weaken $self;
+		my $on_read = sub { $self->_try('I/O watcher', $self->{io}{$fd}{cb}, 0) };
+		my $on_write = sub { $self->_try('I/O watcher', $self->{io}{$fd}{cb}, 1) };
 		my $w = $io->{watcher} = IO::Async::Handle->new(
 			handle => $handle,
-			on_read_ready => sub { $self->_io($fd, 0) },
-			on_write_ready => sub { $self->_io($fd, 1) },
+			on_read_ready => $on_read,
+			on_write_ready => $on_write,
 			want_readready => $read,
 			want_writeready => $write,
 		);
@@ -132,18 +141,12 @@ sub _id {
 	return $id;
 }
 
-sub _io {
-	my ($self, $fd, $writable) = @_;
-	my $io = $self->{io}{$fd};
-	#warn "-- Event fired for IO watcher $fd\n" if DEBUG;
-	$self->_sandbox($writable ? 'Write' : 'Read', $io->{cb}, $writable);
-}
-
 sub _loop { shift->{loop} ||= IO::Async::Loop->new }
 
-sub _sandbox {
-	my ($self, $event, $cb) = (shift, shift, shift);
-	eval { $self->$cb(@_); 1 } or $self->emit(error => "$event failed: $@");
+sub _next {
+	my $self = shift;
+	delete $self->{next_timer};
+	while (my $cb = shift @{$self->{next_tick}}) { $self->$cb }
 }
 
 sub _timer {
@@ -151,19 +154,20 @@ sub _timer {
 	
 	my $id = $self->_id;
 	weaken $self;
+	my $on_expire = sub {
+		my $w = shift;
+		if ($recurring) {
+			$w->start;
+		} else {
+			$w->remove_from_parent;
+			delete $self->{timers}{$id};
+		}
+		#warn "-- Event fired for timer $id\n" if DEBUG;
+		$self->_try('Timer', $cb);
+	};
 	my $w = $self->{timers}{$id}{watcher} = IO::Async::Timer::Countdown->new(
 		delay => $after,
-		on_expire => sub {
-			my $w = shift;
-			if ($recurring) {
-				$w->start;
-			} else {
-				$w->remove_from_parent;
-				delete $self->{timers}{$id};
-			}
-			#warn "-- Event fired for timer $id\n" if DEBUG;
-			$self->_sandbox("Timer $id", $cb);
-		},
+		on_expire => $on_expire,
 	)->start;
 	$self->_loop->add($w);
 	
@@ -175,6 +179,11 @@ sub _timer {
 	return $id;
 }
 
+sub _try {
+	my ($self, $what, $cb) = (shift, shift, shift);
+	eval { $self->$cb(@_); 1 } or $self->emit(error => "$what failed: $@");
+}
+
 =head1 NAME
 
 Mojo::Reactor::IOAsync - IO::Async backend for Mojo::Reactor
@@ -185,18 +194,26 @@ Mojo::Reactor::IOAsync - IO::Async backend for Mojo::Reactor
 
   # Watch if handle becomes readable or writable
   my $reactor = Mojo::Reactor::IOAsync->new;
-  $reactor->io($handle => sub {
+  $reactor->io($first => sub {
     my ($reactor, $writable) = @_;
-    say $writable ? 'Handle is writable' : 'Handle is readable';
+    say $writable ? 'First handle is writable' : 'First handle is readable';
   });
 
   # Change to watching only if handle becomes writable
-  $reactor->watch($handle, 0, 1);
+  $reactor->watch($first, 0, 1);
+
+  # Turn file descriptor into handle and watch if it becomes readable
+  my $second = IO::Handle->new_from_fd($fd, 'r');
+  $reactor->io($second => sub {
+    my ($reactor, $writable) = @_;
+    say $writable ? 'Second handle is writable' : 'Second handle is readable';
+  })->watch($second, 1, 0);
 
   # Add a timer
   $reactor->timer(15 => sub {
     my $reactor = shift;
-    $reactor->remove($handle);
+    $reactor->remove($first);
+    $reactor->remove($second);
     say 'Timeout!';
   });
 
@@ -243,6 +260,12 @@ Restart timer. Note that this method requires an active timer.
 Watch handle for I/O events, invoking the callback whenever handle becomes
 readable or writable.
 
+  # Callback will be invoked twice if handle becomes readable and writable
+  $reactor->io($handle => sub {
+    my ($reactor, $writable) = @_;
+    say $writable ? 'Handle is writable' : 'Handle is readable';
+  });
+
 =head2 is_running
 
   my $bool = $reactor->is_running;
@@ -255,6 +278,13 @@ Check if reactor is running.
 
 Construct a new L<Mojo::Reactor::IOAsync> object.
 
+=head2 next_tick
+
+  my $undef = $reactor->next_tick(sub {...});
+
+Invoke callback as soon as possible, but not before returning or other
+callbacks that have been registered with this method, always returns C<undef>.
+
 =head2 one_tick
 
   $reactor->one_tick;
@@ -262,6 +292,11 @@ Construct a new L<Mojo::Reactor::IOAsync> object.
 Run reactor until an event occurs or no events are being watched anymore. Note
 that this method can recurse back into the reactor, so you need to be careful.
 
+  # Don't block longer than 0.5 seconds
+  my $id = $reactor->timer(0.5 => sub {});
+  $reactor->one_tick;
+  $reactor->remove($id);
+
 =head2 recurring
 
   my $id = $reactor->recurring(0.25 => sub {...});
@@ -289,6 +324,9 @@ Remove all handles and timers.
 Start watching for I/O and timer events, this will block until L</"stop"> is
 called or no events are being watched anymore. See L</"CAVEATS">.
 
+  # Start reactor only if it is not running already
+  $reactor->start unless $reactor->is_running;
+
 =head2 stop
 
   $reactor->stop;
@@ -309,6 +347,18 @@ seconds.
 Change I/O events to watch handle for with true and false values. Note that
 this method requires an active I/O watcher.
 
+  # Watch only for readable events
+  $reactor->watch($handle, 1, 0);
+
+  # Watch only for writable events
+  $reactor->watch($handle, 0, 1);
+
+  # Watch for readable and writable events
+  $reactor->watch($handle, 1, 1);
+
+  # Pause watching for events
+  $reactor->watch($handle, 0, 0);
+
 =head1 CAVEATS
 
 When using L<Mojo::IOLoop> with L<IO::Async>, the event loop must be controlled
